@@ -22,7 +22,6 @@ from datadog_lambda.constants import (
     TraceContextSource,
     XraySubsegment,
     Headers,
-    TraceHeader,
 )
 from datadog_lambda.metric import (
     flush_stats,
@@ -44,6 +43,7 @@ from datadog_lambda.tracing import (
     InferredSpanInfo,
     is_authorizer_response,
     tracer,
+    propagator,
 )
 from datadog_lambda.trigger import (
     extract_trigger_tags,
@@ -60,6 +60,7 @@ DD_FLUSH_TO_LOG = "DD_FLUSH_TO_LOG"
 DD_LOGS_INJECTION = "DD_LOGS_INJECTION"
 DD_MERGE_XRAY_TRACES = "DD_MERGE_XRAY_TRACES"
 AWS_LAMBDA_FUNCTION_NAME = "AWS_LAMBDA_FUNCTION_NAME"
+DD_LOCAL_TEST = "DD_LOCAL_TEST"
 DD_TRACE_EXTRACTOR = "DD_TRACE_EXTRACTOR"
 DD_TRACE_MANAGED_SERVICES = "DD_TRACE_MANAGED_SERVICES"
 DD_ENCODE_AUTHORIZER_CONTEXT = "DD_ENCODE_AUTHORIZER_CONTEXT"
@@ -183,6 +184,9 @@ class _LambdaDecorator(object):
             self.min_cold_start_trace_duration = get_env_as_int(
                 DD_MIN_COLD_START_DURATION, 3
             )
+            self.local_testing_mode = os.environ.get(
+                DD_LOCAL_TEST, "false"
+            ).lower() in ("true", "1")
             self.cold_start_trace_skip_lib = [
                 "ddtrace.internal.compat",
                 "ddtrace.filters",
@@ -250,13 +254,7 @@ class _LambdaDecorator(object):
         injected_headers = {}
         source_span = self.inferred_span if self.inferred_span else self.span
         span_context = source_span.context
-        injected_headers[TraceHeader.TRACE_ID] = str(span_context.trace_id)
-        injected_headers[TraceHeader.PARENT_ID] = str(span_context.span_id)
-        sampling_priority = span_context.sampling_priority
-        if sampling_priority is not None:
-            injected_headers[TraceHeader.SAMPLING_PRIORITY] = str(
-                span_context.sampling_priority
-            )
+        propagator.inject(span_context, injected_headers)
         injected_headers[Headers.Parent_Span_Finish_Time] = finish_time_ns
         if request_id is not None:
             injected_headers[Headers.Authorizing_Request_Id] = request_id
@@ -367,7 +365,10 @@ class _LambdaDecorator(object):
 
             if not self.flush_to_log or should_use_extension:
                 flush_stats()
-            if should_use_extension:
+            if should_use_extension and self.local_testing_mode:
+                # when testing locally, the extension does not know when an
+                # invocation completes because it does not have access to the
+                # logs api
                 flush_extension()
 
             if self.encode_authorizer_context and is_authorizer_response(self.response):
